%% Generated by PeakRDL-beam - A free and open-source Erlang module generator
%% https://github.com/joaohf/PeakRDL-beam

-module({{module}}).

-include("{{module}}.hrl").

-export([
    {%- for exp_function in exported_functions|sort %}
    {{exp_function}}{{ "," if not loop.last }}
    {%- endfor %}
]).